#include "PrettyItemDelegate.h"
#include "../ListModel.h"

#include <QFontMetricsF>
#include <QPainter>

using namespace Playlist;

const qreal PrettyItemDelegate::THUMB_HEIGHT = 90.0;
const qreal PrettyItemDelegate::THUMB_WIDTH = 120.0;
const qreal PrettyItemDelegate::MARGIN = 0.0;
const qreal PrettyItemDelegate::MARGINH = 0.0;
const qreal PrettyItemDelegate::MARGINBODY = 0.0;
const qreal PrettyItemDelegate::PADDING = 10.0;

PrettyItemDelegate::PrettyItemDelegate( QObject* parent ) : QStyledItemDelegate( parent ) {

}

PrettyItemDelegate::~PrettyItemDelegate() { }

QSize PrettyItemDelegate::sizeHint( const QStyleOptionViewItem& option, const QModelIndex& index ) const {
    return QSize( 256, THUMB_HEIGHT+1.0);
}

void PrettyItemDelegate::paint( QPainter* painter,
                                const QStyleOptionViewItem& option, const QModelIndex& index ) const {

    int itemType = index.data(ItemTypeRole).toInt();
    if (itemType == ItemTypeVideo) {
        QApplication::style()->drawPrimitive( QStyle::PE_PanelItemViewItem, &option, painter );
        paintBody( painter, option, index );
    } else
        QStyledItemDelegate::paint( painter, option, index );

}

void PrettyItemDelegate::paintBody( QPainter* painter,
                                    const QStyleOptionViewItem& option,
                                    const QModelIndex& index ) const {

    painter->save();
    painter->translate( option.rect.topLeft() );

    QRectF line(0, 0, option.rect.width(), option.rect.height());

    QPalette palette;

    QFont boldFont;
    boldFont.setBold(true);

    bool isActive = index.data( ActiveTrackRole ).toBool();
    if (isActive) {
        // draw the "current track" highlight underneath the text
        if (! (option.state & QStyle::State_Selected) )
            paintActiveOverlay( painter, line.x(), line.y(), line.width(), line.height() );
        painter->setFont(boldFont);
    }
    QFontMetricsF fm(painter->font());
    QFontMetricsF boldMetrics(boldFont);

    // get the video metadata
    VideoPointer videoPointer = index.data( VideoRole ).value<VideoPointer>();
    Video *video = videoPointer.data();

    // thumb
    painter->drawImage(QRect(0, 0, THUMB_WIDTH, THUMB_HEIGHT), video->thumbnail());

    // title
    QString videoTitle = video->title();
    QRectF textBox = line.adjusted(PADDING+THUMB_WIDTH, PADDING, -2 * PADDING, -PADDING);
    textBox = painter->boundingRect( textBox, Qt::AlignLeft | Qt::AlignTop | Qt::TextWordWrap, videoTitle);

    // time
    QString timeString;
    int duration = video->duration();
    if ( duration > 3600 )
        timeString = QTime().addSecs( duration ).toString( "h:mm:ss" );
    else
        timeString = QTime().addSecs( duration ).toString( "m:ss" );
    QSizeF timeStringSize( fm.size( Qt::TextSingleLine, timeString ) );
    QPointF textLoc(PADDING+THUMB_WIDTH, PADDING*2 + textBox.height());
    QRectF timeTextBox( textLoc, timeStringSize);

    // author
    QString authorString = video->author();
    QSizeF authorStringSize( boldMetrics.size( Qt::TextSingleLine, authorString ) );
    textLoc.setX(textLoc.x() + timeStringSize.width() + PADDING);
    QRectF authorTextBox( textLoc , authorStringSize);

    // published date
    QString publishedString = video->published().date().toString(Qt::DefaultLocaleShortDate);
    QSizeF publishedStringSize( fm.size( Qt::TextSingleLine, publishedString ) );
    textLoc.setX(textLoc.x() + authorStringSize.width() + PADDING);
    QRectF publishedTextBox( textLoc , publishedStringSize);

    // text color
    if ( option.state & QStyle::State_Selected ) {
        painter->setPen(palette.color(QPalette::HighlightedText));
    } else {
        painter->setPen(palette.color(QPalette::Text));
    }

    // draw normal text
    painter->drawText(textBox, Qt::AlignLeft | Qt::AlignTop | Qt::TextWordWrap, videoTitle);
    painter->drawText(timeTextBox, Qt::AlignLeft | Qt::AlignTop, timeString);
    painter->drawText(publishedTextBox, Qt::AlignLeft | Qt::AlignTop, publishedString);

    painter->setFont(boldFont);
    painter->setPen(palette.color(QPalette::Dark));
    painter->drawText(authorTextBox, Qt::AlignLeft | Qt::AlignTop, authorString);

    // separator
    painter->setPen(palette.color(QPalette::Midlight));
    painter->drawLine(0, THUMB_HEIGHT, line.width(), THUMB_HEIGHT);

    painter->restore();

}

QPointF PrettyItemDelegate::centerImage( const QPixmap& pixmap, const QRectF& rect ) const {
    qreal pixmapRatio = ( qreal )pixmap.width() / ( qreal )pixmap.height();

    qreal moveByX = 0.0;
    qreal moveByY = 0.0;

    if ( pixmapRatio >= 1 )
        moveByY = ( rect.height() - ( rect.width() / pixmapRatio ) ) / 2.0;
    else
        moveByX = ( rect.width() - ( rect.height() * pixmapRatio ) ) / 2.0;

    return QPointF( moveByX, moveByY );
}

void PrettyItemDelegate::paintActiveOverlay( QPainter *painter, qreal x, qreal y, qreal w, qreal h ) const {

    QPalette palette;
    QColor color2 = palette.color( QPalette::Highlight);
    QColor backgroundColor = palette.color( QPalette::Base);
    float animation = 0.33;
    color2 = QColor::fromHsv(
            animation == 0.0 ? backgroundColor.hue() : color2.hue(),
            (int)(backgroundColor.saturation()*(1.0f-animation)+color2.saturation()*animation),
            (int)(backgroundColor.value()*(1.0f-animation)+color2.value()*animation)
            );
    QColor color1 = QColor::fromHsv(
            color2.hue(),
            (color2.saturation() - 8 > 0) ? color2.saturation() - 8 : 0,
            (color2.value() + 8 < 255) ? color2.value() + 8 : 255
            );
    // painter->drawRect((int) x, (int) y, (int) w, (int) h);
    QRect rect((int) x, (int) y, (int) w, (int) h);
    painter->save();
    painter->setRenderHints (QPainter::Antialiasing, true);
    painter->setBackgroundMode(Qt::OpaqueMode);
    painter->setPen(Qt::NoPen);
    // painter.setOpacity(.75);
    QLinearGradient linearGradient(0, 0, 0, rect.height());
    linearGradient.setColorAt(0.0, color1);
    linearGradient.setColorAt(1.0, color2);
    painter->setBrush(linearGradient); 
    painter->drawRect(rect);
    painter->restore();
}

